@@ -2,8 +2,10 @@ from . import admin  # noqa  # type: ignore
 from . import config  # noqa  # type: ignore
 from . import dotfile  # noqa  # type: ignore
 from . import extensions  # noqa  # type: ignore
+from . import model  # noqa  # type: ignore
 from . import server_log  # noqa  # type: ignore
+from . import service  # noqa  # type: ignore
 from . import session  # noqa  # type: ignore
 from . import session_template  # noqa  # type: ignore
 from . import vfolder  # noqa  # type: ignore
-from . import app, logs, proxy, run  # noqa  # type: ignore
+from . import app, logs, proxy  # noqa  # type: ignore
