@@ -46,6 +46,7 @@ def info(ctx: CLIContext, email: str) -> None:
         user_fields["domain_name"],
         user_fields["groups"],
         user_fields["allowed_client_ip"],
+        user_fields["sudo_session_enabled"],
     ]
     with Session() as session:
         try:
@@ -65,9 +66,67 @@ def info(ctx: CLIContext, email: str) -> None:
     default=None,
     help="Filter users in a specific state (active, inactive, deleted, before-verification).",
 )
-@click.option("-g", "--group", type=str, default=None, help="Filter by group ID.")
-@click.option("--filter", "filter_", default=None, help="Set the query filter expression.")
-@click.option("--order", default=None, help="Set the query ordering expression.")
+@click.option(
+    "-g",
+    "--group",
+    type=str,
+    default=None,
+    help="""
+    Filter by group ID.
+
+    \b
+    EXAMPLE
+        --group "$(backend.ai admin group list | grep 'example-group-name' | awk '{print $1}')"
+    """,
+)
+@click.option(
+    "--filter",
+    "filter_",
+    default=None,
+    help="""
+    Set the query filter expression.
+
+    \b
+    COLUMNS
+        uuid, username, role, email, full_name, need_password_change,
+        status, status_info, created_at, modified_at, domain_name, allowed_client_ip
+
+    \b
+    OPERATORS
+        Binary Operators: ==, !=, <, <=, >, >=, is, isnot, like, ilike(case-insensitive), in, contains
+        Condition Operators: &, |
+        Special Symbol: % (wildcard for like and ilike operators)
+
+    \b
+    EXAMPLE QUERIES
+        --filter 'status == "ACTIVE" & role in ["ADMIN", "SUPERADMIN"]'
+        --filter 'created_at >= "2021-01-01" & created_at < "2023-01-01"'
+        --filter 'email ilike "%@example.com"'
+    """,
+)
+@click.option(
+    "--order",
+    default=None,
+    help="""
+    Set the query ordering expression.
+
+    \b
+    COLUMNS
+        uuid, username, role, email, full_name, need_password_change,
+        status, status_info, created_at, modified_at, domain_name
+
+    \b
+    OPTIONS
+        ascending order (default): (+)column_name
+        descending order: -column_name
+
+    \b
+    EXAMPLE
+        --order 'uuid'
+        --order '+uuid'
+        --order '-created_at'
+    """,
+)
 @click.option("--offset", default=0, help="The index of the current page start for pagination.")
 @click.option("--limit", type=int, default=None, help="The page size for pagination.")
 def list(ctx: CLIContext, status, group, filter_, order, offset, limit) -> None:
@@ -88,6 +147,7 @@ def list(ctx: CLIContext, status, group, filter_, order, offset, limit) -> None:
         user_fields["domain_name"],
         user_fields["groups"],
         user_fields["allowed_client_ip"],
+        user_fields["sudo_session_enabled"],
     ]
     try:
         with Session() as session:
@@ -134,17 +194,30 @@ def list(ctx: CLIContext, status, group, filter_, order, offset, limit) -> None:
 @click.option(
     "--need-password-change",
     is_flag=True,
-    help="Flag indicate that user needs to change password. "
-    "Useful when admin manually create password.",
+    help=(
+        "Flag indicate that user needs to change password. "
+        "Useful when admin manually create password."
+    ),
 )
 @click.option(
     "--allowed-ip",
     type=list_expr,
     default=None,
-    help="Allowed client IP. IPv4 and IPv6 are allowed. CIDR type is recommended. "
-    '(e.g., --allowed-ip "127.0.0.1","127.0.0.2",...)',
+    help=(
+        "Allowed client IP. IPv4 and IPv6 are allowed. CIDR type is recommended. "
+        '(e.g., --allowed-ip "127.0.0.1","127.0.0.2",...)'
+    ),
 )
 @click.option("--description", type=str, default="", help="Description of the user.")
+@click.option(
+    "--sudo-session-enabled",
+    is_flag=True,
+    default=False,
+    help=(
+        "Enable passwordless sudo for a user inside a compute session. "
+        "Note that this feature does not automatically install sudo for the session."
+    ),
+)
 def add(
     ctx: CLIContext,
     domain_name,
@@ -157,6 +230,7 @@ def add(
     need_password_change,
     allowed_ip,
     description,
+    sudo_session_enabled,
 ):
     """
     Add new user. A user must belong to a domain, so DOMAIN_NAME should be provided.
@@ -179,6 +253,7 @@ def add(
                 need_password_change=need_password_change,
                 allowed_client_ip=allowed_ip,
                 description=description,
+                sudo_session_enabled=sudo_session_enabled,
             )
         except Exception as e:
             ctx.output.print_mutation_error(
@@ -222,17 +297,30 @@ def add(
 @click.option(
     "--need-password-change",
     is_flag=True,
-    help="Flag indicate that user needs to change password. "
-    "Useful when admin manually create password.",
+    help=(
+        "Flag indicate that user needs to change password. "
+        "Useful when admin manually create password."
+    ),
 )
 @click.option(
     "--allowed-ip",
     type=list_expr,
     default=None,
-    help="Allowed client IP. IPv4 and IPv6 are allowed. CIDR type is recommended. "
-    '(e.g., --allowed-ip "127.0.0.1","127.0.0.2",...)',
+    help=(
+        "Allowed client IP. IPv4 and IPv6 are allowed. CIDR type is recommended. "
+        '(e.g., --allowed-ip "127.0.0.1","127.0.0.2",...)'
+    ),
 )
 @click.option("--description", type=str, default="", help="Description of the user.")
+@click.option(
+    "--sudo-session-enabled",
+    is_flag=True,
+    default=False,
+    help=(
+        "Enable passwordless sudo for a user inside a compute session. "
+        "Note that this feature does not automatically install sudo for the session."
+    ),
+)
 def update(
     ctx: CLIContext,
     email,
@@ -245,10 +333,12 @@ def update(
     need_password_change,
     allowed_ip,
     description,
+    sudo_session_enabled,
 ):
     """
     Update an existing user.
 
+    \b
     EMAIL: Email of user to update.
     """
     with Session() as session:
@@ -264,6 +354,7 @@ def update(
                 need_password_change=need_password_change,
                 allowed_client_ip=allowed_ip,
                 description=description,
+                sudo_session_enabled=sudo_session_enabled,
             )
         except Exception as e:
             ctx.output.print_mutation_error(
@@ -294,6 +385,7 @@ def delete(ctx: CLIContext, email):
     """
     Inactivate an existing user.
 
+    \b
     EMAIL: Email of user to inactivate.
     """
     with Session() as session:
@@ -328,14 +420,17 @@ def delete(ctx: CLIContext, email):
     "--purge-shared-vfolders",
     is_flag=True,
     default=False,
-    help="Delete user's all virtual folders. "
-    "If False, shared folders will not be deleted "
-    "and migrated the ownership to the requested admin.",
+    help=(
+        "Delete user's all virtual folders. "
+        "If False, shared folders will not be deleted "
+        "and migrated the ownership to the requested admin."
+    ),
 )
 def purge(ctx: CLIContext, email, purge_shared_vfolders):
     """
     Delete an existing user. This action cannot be undone.
 
+    \b
     NAME: Name of a domain to delete.
     """
     with Session() as session:
