@@ -6,18 +6,32 @@ from pathlib import Path
 import click
 import humanize
 from tabulate import tabulate
-from tqdm import tqdm
 
 from ai.backend.cli.interaction import ask_yn
 from ai.backend.cli.main import main
 from ai.backend.cli.types import ExitCode
 from ai.backend.client.config import DEFAULT_CHUNK_SIZE, APIConfig
+from ai.backend.client.func.vfolder import _default_list_fields
 from ai.backend.client.session import Session
 
 from ..compat import asyncio_run
 from ..session import AsyncSession
-from .params import ByteSizeParamCheckType, ByteSizeParamType, CommaSeparatedKVListParamType
-from .pretty import print_done, print_error, print_fail, print_info, print_wait, print_warn
+from .extensions import pass_ctx_obj
+from .params import (
+    ByteSizeParamCheckType,
+    ByteSizeParamType,
+    CommaSeparatedKVListParamType,
+)
+from .pretty import (
+    ProgressViewer,
+    print_done,
+    print_error,
+    print_fail,
+    print_info,
+    print_wait,
+    print_warn,
+)
+from .types import CLIContext
 
 
 @main.group()
@@ -66,8 +80,10 @@ def list_allowed_types():
     "host_path",
     type=bool,
     is_flag=True,
-    help="Treats HOST as a mount point of unmanaged virtual folder. "
-    "This option can only be used by Admin or Superadmin.",
+    help=(
+        "Treats HOST as a mount point of unmanaged virtual folder. "
+        "This option can only be used by Admin or Superadmin."
+    ),
 )
 @click.option(
     "-m",
@@ -75,9 +91,11 @@ def list_allowed_types():
     metavar="USAGE_MODE",
     type=str,
     default="general",
-    help='Purpose of the folder. Normal folders are usually set to "general". '
-    'Available options: "general", "data" (provides data to users), '
-    'and "model" (provides pre-trained models).',
+    help=(
+        'Purpose of the folder. Normal folders are usually set to "general". '
+        'Available options: "general", "data" (provides data to users), '
+        'and "model" (provides pre-trained models).'
+    ),
 )
 @click.option(
     "-p",
@@ -85,9 +103,11 @@ def list_allowed_types():
     metavar="PERMISSION",
     type=str,
     default="rw",
-    help="Folder's innate permission. "
-    'Group folders can be shared as read-only by setting this option to "ro".'
-    "Invited folders override this setting by its own invitation permission.",
+    help=(
+        "Folder's innate permission. "
+        'Group folders can be shared as read-only by setting this option to "ro". '
+        "Invited folders override this setting by its own invitation permission. "
+    ),
 )
 @click.option(
     "-q",
@@ -95,9 +115,11 @@ def list_allowed_types():
     metavar="QUOTA",
     type=ByteSizeParamCheckType(),
     default="0",
-    help="Quota of the virtual folder. "
-    "(Use 'm' for megabytes, 'g' for gigabytes, and etc.) "
-    "Default is maximum amount possible.",
+    help=(
+        "Quota of the virtual folder. "
+        "(Use 'm' for megabytes, 'g' for gigabytes, and etc.) "
+        "Default is maximum amount possible."
+    ),
 )
 @click.option(
     "--cloneable",
@@ -146,6 +168,7 @@ def create(name, host, group, host_path, usage_mode, permission, quota, cloneabl
 def delete(name):
     """Delete the given virtual folder. This operation is irreversible!
 
+    \b
     NAME: Name of a virtual folder.
     """
     with Session() as session:
@@ -166,6 +189,7 @@ def rename(old_name, new_name):
     and the new name must be unique among all your accessible vfolders
     including the shared ones.
 
+    \b
     OLD_NAME: The current name of a virtual folder.
     NEW_NAME: The new name of a virtual folder.
     """
@@ -183,6 +207,7 @@ def rename(old_name, new_name):
 def info(name):
     """Show the information of the given virtual folder.
 
+    \b
     NAME: Name of a virtual folder.
     """
     with Session() as session:
@@ -212,30 +237,41 @@ def info(name):
     "--base-dir",
     type=Path,
     default=None,
-    help="Set the parent directory from where the file is uploaded.  "
-    "[default: current working directry]",
+    help=(
+        "The local parent directory which contains the file to be uploaded. "
+        "[default: current working directory]"
+    ),
+)
+@click.option(
+    "-r",
+    "--recursive",
+    is_flag=True,
+    help="Upload the given directory recursively.",
 )
 @click.option(
     "--chunk-size",
     type=ByteSizeParamType(),
     default=humanize.naturalsize(DEFAULT_CHUNK_SIZE, binary=True, gnu=True),
-    help='Transfer the file with the given chunk size with binary suffixes (e.g., "16m"). '
-    "Set this between 8 to 64 megabytes for high-speed disks (e.g., SSD RAID) "
-    "and networks (e.g., 40 GbE) for the maximum throughput.",
+    help=(
+        "Transfer the file with the given chunk size with binary suffixes (e.g.,"
+        ' "16m"). Set this between 8 to 64 megabytes for high-speed disks (e.g., SSD'
+        " RAID) and networks (e.g., 40 GbE) for the maximum throughput."
+    ),
 )
 @click.option(
     "--override-storage-proxy",
     type=CommaSeparatedKVListParamType(),
     default=None,
-    help="Overrides storage proxy address. "
-    'The value must shape like "X1=Y1,X2=Y2...". '
-    "Each Yn address must at least include the IP address "
-    "or the hostname and may include the protocol part and the port number to replace.",
+    help=(
+        'Overrides storage proxy address. The value must shape like "X1=Y1,X2=Y2...".'
+        " Each Yn address must at least include the IP address or the hostname and may"
+        " include the protocol part and the port number to replace."
+    ),
 )
-def upload(name, filenames, base_dir, chunk_size, override_storage_proxy):
+def upload(name, filenames, base_dir, recursive, chunk_size, override_storage_proxy):
     """
     TUS Upload a file to the virtual folder from the current working directory.
-    The files with the same names will be overwirtten.
+    The files with the same names will be overwritten.
 
     \b
     NAME: Name of a virtual folder.
@@ -246,6 +282,7 @@ def upload(name, filenames, base_dir, chunk_size, override_storage_proxy):
             session.VFolder(name).upload(
                 filenames,
                 basedir=base_dir,
+                recursive=recursive,
                 chunk_size=chunk_size,
                 show_progress=True,
                 address_map=override_storage_proxy
@@ -265,25 +302,30 @@ def upload(name, filenames, base_dir, chunk_size, override_storage_proxy):
     "--base-dir",
     type=Path,
     default=None,
-    help="Set the parent directory from where the file is uploaded.  "
-    "[default: current working directry]",
+    help=(
+        "The local parent directory which will contain the downloaded file.  "
+        "[default: current working directory]"
+    ),
 )
 @click.option(
     "--chunk-size",
     type=ByteSizeParamType(),
     default=humanize.naturalsize(DEFAULT_CHUNK_SIZE, binary=True, gnu=True),
-    help='Transfer the file with the given chunk size with binary suffixes (e.g., "16m"). '
-    "Set this between 8 to 64 megabytes for high-speed disks (e.g., SSD RAID) "
-    "and networks (e.g., 40 GbE) for the maximum throughput.",
+    help=(
+        "Transfer the file with the given chunk size with binary suffixes (e.g.,"
+        ' "16m"). Set this between 8 to 64 megabytes for high-speed disks (e.g., SSD'
+        " RAID) and networks (e.g., 40 GbE) for the maximum throughput."
+    ),
 )
 @click.option(
     "--override-storage-proxy",
     type=CommaSeparatedKVListParamType(),
     default=None,
-    help="Overrides storage proxy address. "
-    'The value must shape like "X1=Y1,X2=Y2...". '
-    "Each Yn address must at least include the IP address "
-    "or the hostname and may include the protocol part and the port number to replace.",
+    help=(
+        'Overrides storage proxy address. The value must shape like "X1=Y1,X2=Y2...".'
+        " Each Yn address must at least include the IP address or the hostname and may"
+        " include the protocol part and the port number to replace."
+    ),
 )
 @click.option(
     "--max-retries",
@@ -294,7 +336,7 @@ def upload(name, filenames, base_dir, chunk_size, override_storage_proxy):
 def download(name, filenames, base_dir, chunk_size, override_storage_proxy, max_retries):
     """
     Download a file from the virtual folder to the current working directory.
-    The files with the same names will be overwirtten.
+    The files with the same names will be overwritten.
 
     \b
     NAME: Name of a virtual folder.
@@ -322,7 +364,7 @@ def download(name, filenames, base_dir, chunk_size, override_storage_proxy, max_
 @click.argument("filename", type=Path)
 def request_download(name, filename):
     """
-    Request JWT-formated download token for later use.
+    Request JWT-formatted download token for later use.
 
     \b
     NAME: Name of a virtual folder.
@@ -342,6 +384,7 @@ def request_download(name, filename):
 def cp(filenames):
     """An scp-like shortcut for download/upload commands.
 
+    \b
     FILENAMES: Paths of the files to operate on. The last one is the target while all
                others are the sources.  Either source paths or the target path should
                be prefixed with "<vfolder-name>:" like when using the Linux scp
@@ -361,7 +404,11 @@ def cp(filenames):
     help="Make missing parents of this path as needed",
 )
 @click.option(
-    "-e", "--exist-ok", default=False, is_flag=True, help="Skip an error caused by file not found"
+    "-e",
+    "--exist-ok",
+    default=False,
+    is_flag=True,
+    help="Skip an error caused by file not found",
 )
 def mkdir(name, path, parents, exist_ok):
     """Create an empty directory in the virtual folder.
@@ -481,7 +528,7 @@ def ls(name, path):
                 mtime = mdt.strftime("%b %d %Y %H:%M:%S")
                 row = [file["filename"], file["size"], mtime, file["mode"]]
                 table.append(row)
-            print_done("Retrived.")
+            print_done("Retrieved.")
             print(tabulate(table, headers=headers))
         except Exception as e:
             print_error(e)
@@ -642,8 +689,9 @@ def unshare(name, emails):
     help="The ID of the person who wants to leave (the person who shared the vfolder).",
 )
 def leave(name, shared_user_uuid):
-    """Leave the shared virutal folder.
+    """Leave the shared virtual folder.
 
+    \b
     NAME: Name of a virtual folder
     """
     with Session() as session:
@@ -673,7 +721,7 @@ def leave(name, shared_user_uuid):
     metavar="USAGE_MODE",
     type=str,
     default="general",
-    help="Purpose of the cloned virtual folder. " "Default value is 'general'.",
+    help="Purpose of the cloned virtual folder. Default value is 'general'.",
 )
 @click.option(
     "-p",
@@ -681,7 +729,7 @@ def leave(name, shared_user_uuid):
     metavar="PERMISSION",
     type=str,
     default="rw",
-    help="Cloned virtual folder's permission. " "Default value is 'rw'.",
+    help="Cloned virtual folder's permission. Default value is 'rw'.",
 )
 def clone(name, target_name, target_host, usage_mode, permission):
     """Clone a virtual folder.
@@ -711,34 +759,38 @@ def clone(name, target_name, target_host, usage_mode, permission):
             print_error(e)
             sys.exit(ExitCode.FAILURE)
 
+    # NOTE: Tracking the progress from the storage-proxy is not supported yet. (See #1033)
     async def clone_vfolder_tracker(bgtask_id):
-        print_wait(
-            "Cloning the vfolder... "
-            "(This may take a while depending on its size and number of files!)",
-        )
         async with AsyncSession() as session:
             try:
                 bgtask = session.BackgroundTask(bgtask_id)
                 completion_msg_func = lambda: print_done("Cloning the vfolder is complete.")
-                async with bgtask.listen_events() as response:
-                    # TODO: get the unit of progress from response
-                    with tqdm(unit="bytes", disable=True) as pbar:
-                        async for ev in response:
-                            data = json.loads(ev.data)
-                            if ev.event == "bgtask_updated":
+                async with (
+                    bgtask.listen_events() as response,
+                    ProgressViewer(
+                        "Cloning the vfolder... "
+                        "(This may take a while depending on its size and number of files!)",
+                    ) as viewer,
+                ):
+                    async for ev in response:
+                        data = json.loads(ev.data)
+                        if ev.event == "bgtask_updated":
+                            if viewer.tqdm is None:
+                                pbar = await viewer.to_tqdm()
+                            else:
                                 pbar.total = data["total_progress"]
                                 pbar.write(data["message"])
                                 pbar.update(data["current_progress"] - pbar.n)
-                            elif ev.event == "bgtask_failed":
-                                error_msg = data["message"]
-                                completion_msg_func = lambda: print_fail(
-                                    f"Error during the operation: {error_msg}",
-                                )
-                            elif ev.event == "bgtask_cancelled":
-                                completion_msg_func = lambda: print_warn(
-                                    "The operation has been cancelled in the middle. "
-                                    "(This may be due to server shutdown.)",
-                                )
+                        elif ev.event == "bgtask_failed":
+                            error_msg = data["message"]
+                            completion_msg_func = lambda: print_fail(
+                                f"Error during the operation: {error_msg}",
+                            )
+                        elif ev.event == "bgtask_cancelled":
+                            completion_msg_func = lambda: print_warn(
+                                "The operation has been cancelled in the middle. "
+                                "(This may be due to server shutdown.)",
+                            )
             finally:
                 completion_msg_func()
 
@@ -751,14 +803,20 @@ def clone(name, target_name, target_host, usage_mode, permission):
 @vfolder.command()
 @click.argument("name", type=str)
 @click.option(
-    "-p", "--permission", type=str, metavar="PERMISSION", help="Folder's innate permission."
+    "-p",
+    "--permission",
+    type=str,
+    metavar="PERMISSION",
+    help="Folder's innate permission.",
 )
 @click.option(
     "--set-cloneable",
     type=bool,
     metavar="BOOLEXPR",
-    help="A boolean-interpretable string whether a virtual folder can be cloned. "
-    "If not set, the cloneable property is not changed.",
+    help=(
+        "A boolean-interpretable string whether a virtual folder can be cloned. "
+        "If not set, the cloneable property is not changed."
+    ),
 )
 def update_options(name, permission, set_cloneable):
     """Update an existing virtual folder.
@@ -781,3 +839,243 @@ def update_options(name, permission, set_cloneable):
         except Exception as e:
             print_error(e)
             sys.exit(ExitCode.FAILURE)
+
+
+@vfolder.command()
+@pass_ctx_obj
+@click.option(
+    "--filter",
+    "filter_",
+    default=None,
+    help="""\b
+    Set the query filter expression.
+
+    \b
+    COLUMNS
+        host, name, created_at, creator,
+        ownership_type (UESR, GROUP),
+        status (READY, PERFORMING, CLONING, DELETING, MOUNTED),
+        permission (READ_ONLY, READ_WRITE, RW_DELETE, OWNER_PERM)
+
+    \b
+    OPERATORS
+        Binary Operators: ==, !=, <, <=, >, >=, is, isnot, like, ilike(case-insensitive), in, contains
+        Condition Operators: &, |
+        Special Symbol: % (wildcard for like and ilike operators)
+
+    \b
+    EXAMPLE QUERIES
+        --filter 'status == "READY" & permission in ["READ_ONLY", "READ_WRITE"]'
+        --filter 'created_at >= "2021-01-01" & created_at < "2023-01-01"'
+        --filter 'creator ilike "%@example.com"'
+
+    \b
+    """,
+)
+@click.option(
+    "--order",
+    default=None,
+    help="""\b
+    Set the query ordering expression.
+
+    \b
+    COLUMNS
+        host, name, created_at, creator, ownership_type, status, permission
+
+    \b
+    OPTIONS
+        ascending order (default): (+)column_name
+        descending order: -column_name
+
+    \b
+    EXAMPLE
+        --order 'host'
+        --order '+host'
+        --order '-created_at'
+
+    \b
+    """,
+)
+@click.option("--offset", default=0, help="The index of the current page start for pagination.")
+@click.option("--limit", type=int, default=None, help="The page size for pagination.")
+def list_own(ctx: CLIContext, filter_, order, offset, limit) -> None:
+    """
+    List own virtual folders.
+    """
+    try:
+        with Session() as session:
+            fetch_func = lambda pg_offset, pg_size: session.VFolder.paginated_own_list(
+                fields=_default_list_fields,
+                page_offset=pg_offset,
+                page_size=pg_size,
+                filter=filter_,
+                order=order,
+            )
+            ctx.output.print_paginated_list(
+                fetch_func,
+                initial_page_offset=offset,
+                page_size=limit,
+            )
+    except Exception as e:
+        ctx.output.print_error(e)
+        sys.exit(ExitCode.FAILURE)
+
+
+@vfolder.command()
+@pass_ctx_obj
+@click.option(
+    "--filter",
+    "filter_",
+    default=None,
+    help="""\b
+    Set the query filter expression.
+
+    \b
+    COLUMNS
+        host, name, created_at, creator,
+        ownership_type (UESR, GROUP),
+        status (READY, PERFORMING, CLONING, DELETING, MOUNTED),
+        permission (READ_ONLY, READ_WRITE, RW_DELETE, OWNER_PERM)
+
+    \b
+    OPERATORS
+        Binary Operators: ==, !=, <, <=, >, >=, is, isnot, like, ilike(case-insensitive), in, contains
+        Condition Operators: &, |
+        Special Symbol: % (wildcard for like and ilike operators)
+
+    \b
+    EXAMPLE QUERIES
+        --filter 'status == "READY" & permission in ["READ_ONLY", "READ_WRITE"]'
+        --filter 'created_at >= "2021-01-01" & created_at < "2023-01-01"'
+        --filter 'creator ilike "%@example.com"'
+
+    \b
+    """,
+)
+@click.option(
+    "--order",
+    default=None,
+    help="""\b
+    Set the query ordering expression.
+
+    \b
+    COLUMNS
+        host, name, created_at, creator, ownership_type, status, permission
+
+    \b
+    OPTIONS
+        ascending order (default): (+)column_name
+        descending order: -column_name
+
+    \b
+    EXAMPLE
+        --order 'host'
+        --order '+host'
+        --order '-created_at'
+
+    \b
+    """,
+)
+@click.option("--offset", default=0, help="The index of the current page start for pagination.")
+@click.option("--limit", type=int, default=None, help="The page size for pagination.")
+def list_invited(ctx: CLIContext, filter_, order, offset, limit) -> None:
+    """
+    List invited virtual folders.
+    """
+    try:
+        with Session() as session:
+            fetch_func = lambda pg_offset, pg_size: session.VFolder.paginated_invited_list(
+                fields=_default_list_fields,
+                page_offset=pg_offset,
+                page_size=pg_size,
+                filter=filter_,
+                order=order,
+            )
+            ctx.output.print_paginated_list(
+                fetch_func,
+                initial_page_offset=offset,
+                page_size=limit,
+            )
+    except Exception as e:
+        ctx.output.print_error(e)
+        sys.exit(ExitCode.FAILURE)
+
+
+@vfolder.command()
+@pass_ctx_obj
+@click.option(
+    "--filter",
+    "filter_",
+    default=None,
+    help="""\b
+    Set the query filter expression.
+
+    \b
+    COLUMNS
+        host, name, created_at, creator,
+        ownership_type (UESR, GROUP),
+        status (READY, PERFORMING, CLONING, DELETING, MOUNTED),
+        permission (READ_ONLY, READ_WRITE, RW_DELETE, OWNER_PERM)
+
+    \b
+    OPERATORS
+        Binary Operators: ==, !=, <, <=, >, >=, is, isnot, like, ilike(case-insensitive), in, contains
+        Condition Operators: &, |
+        Special Symbol: % (wildcard for like and ilike operators)
+
+    \b
+    EXAMPLE QUERIES
+        --filter 'status == "READY" & permission in ["READ_ONLY", "READ_WRITE"]'
+        --filter 'created_at >= "2021-01-01" & created_at < "2023-01-01"'
+        --filter 'creator ilike "%@example.com"'
+
+    \b
+    """,
+)
+@click.option(
+    "--order",
+    default=None,
+    help="""\b
+    Set the query ordering expression.
+
+    \b
+    COLUMNS
+        host, name, created_at, creator, ownership_type, status, permission
+
+    \b
+    OPTIONS
+        ascending order (default): (+)column_name
+        descending order: -column_name
+
+    \b
+    EXAMPLE
+        --order 'host'
+        --order '+host'
+        --order '-created_at'
+
+    \b
+    """,
+)
+@click.option("--offset", default=0, help="The index of the current page start for pagination.")
+@click.option("--limit", type=int, default=None, help="The page size for pagination.")
+def list_project(ctx: CLIContext, filter_, order, offset, limit) -> None:
+    """
+    List project virtual folders.
+    """
+    try:
+        with Session() as session:
+            fetch_func = lambda pg_offset, pg_size: session.VFolder.paginated_project_list(
+                fields=_default_list_fields,
+                page_offset=pg_offset,
+                page_size=pg_size,
+                filter=filter_,
+                order=order,
+            )
+            ctx.output.print_paginated_list(
+                fetch_func,
+                initial_page_offset=offset,
+                page_size=limit,
+            )
+    except Exception as e:
+        ctx.output.print_error(e)
+        sys.exit(ExitCode.FAILURE)
