@@ -1,6 +1,6 @@
 import json
 import textwrap
-from typing import Iterable, Mapping, Sequence
+from typing import Iterable, Mapping, Optional, Sequence
 
 from ai.backend.client.output.fields import scaling_group_fields
 from ai.backend.client.output.types import FieldSpec
@@ -15,22 +15,28 @@ _default_list_fields = (
     scaling_group_fields["name"],
     scaling_group_fields["description"],
     scaling_group_fields["is_active"],
+    scaling_group_fields["is_public"],
     scaling_group_fields["created_at"],
     scaling_group_fields["driver"],
     scaling_group_fields["scheduler"],
     scaling_group_fields["use_host_network"],
+    scaling_group_fields["wsproxy_addr"],
+    scaling_group_fields["wsproxy_api_token"],
 )
 
 _default_detail_fields = (
     scaling_group_fields["name"],
     scaling_group_fields["description"],
     scaling_group_fields["is_active"],
+    scaling_group_fields["is_public"],
     scaling_group_fields["created_at"],
     scaling_group_fields["driver"],
     scaling_group_fields["driver_opts"],
     scaling_group_fields["scheduler"],
     scaling_group_fields["scheduler_opts"],
     scaling_group_fields["use_host_network"],
+    scaling_group_fields["wsproxy_addr"],
+    scaling_group_fields["wsproxy_api_token"],
 )
 
 
@@ -73,15 +79,13 @@ class ScalingGroup(BaseFunction):
         List available scaling groups for the current user,
         considering the user, the user's domain, and the designated user group.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             query($is_active: Boolean) {
                 scaling_groups(is_active: $is_active) {
                     $fields
                 }
             }
-        """
-        )
+        """)
         query = query.replace("$fields", " ".join(f.field_ref for f in fields))
         variables = {"is_active": None}
         data = await api_session.get().Admin._query(query, variables)
@@ -100,13 +104,11 @@ class ScalingGroup(BaseFunction):
         :param name: Name of the scaling group.
         :param fields: Additional per-scaling-group query fields.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             query($name: String) {
                 scaling_group(name: $name) {$fields}
             }
-        """
-        )
+        """)
         query = query.replace("$fields", " ".join(f.field_ref for f in fields))
         variables = {"name": name}
         data = await api_session.get().Admin._query(query, variables)
@@ -119,25 +121,26 @@ class ScalingGroup(BaseFunction):
         name: str,
         description: str = "",
         is_active: bool = True,
+        is_public: bool = True,
         driver: str = None,
         driver_opts: Mapping[str, str] = None,
         scheduler: str = None,
         scheduler_opts: Mapping[str, str] = None,
         use_host_network: bool = False,
+        wsproxy_addr: Optional[str] = None,
+        wsproxy_api_token: Optional[str] = None,
         fields: Iterable[FieldSpec | str] = None,
     ) -> dict:
         """
         Creates a new scaling group with the given options.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($name: String!, $input: CreateScalingGroupInput!) {
                 create_scaling_group(name: $name, props: $input) {
                     ok msg scaling_group {$fields}
                 }
             }
-        """
-        )
+        """)
         resolved_fields = resolve_fields(
             fields, scaling_group_fields, (scaling_group_fields["name"],)
         )
@@ -147,11 +150,14 @@ class ScalingGroup(BaseFunction):
             "input": {
                 "description": description,
                 "is_active": is_active,
+                "is_public": is_public,
                 "driver": driver,
                 "driver_opts": json.dumps(driver_opts),
                 "scheduler": scheduler,
                 "scheduler_opts": json.dumps(scheduler_opts),
                 "use_host_network": use_host_network,
+                "wsproxy_addr": wsproxy_addr,
+                "wsproxy_api_token": wsproxy_api_token,
             },
         }
         data = await api_session.get().Admin._query(query, variables)
@@ -164,25 +170,26 @@ class ScalingGroup(BaseFunction):
         name: str,
         description: str = "",
         is_active: bool = True,
+        is_public: bool = True,
         driver: str = None,
         driver_opts: Mapping[str, str] = None,
         scheduler: str = None,
         scheduler_opts: Mapping[str, str] = None,
         use_host_network: bool = False,
+        wsproxy_addr: Optional[str] = None,
+        wsproxy_api_token: Optional[str] = None,
         fields: Iterable[FieldSpec | str] = None,
     ) -> dict:
         """
         Update existing scaling group.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($name: String!, $input: ModifyScalingGroupInput!) {
                 modify_scaling_group(name: $name, props: $input) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         resolved_fields = resolve_fields(
             fields, scaling_group_fields, (scaling_group_fields["name"],)
         )
@@ -192,11 +199,14 @@ class ScalingGroup(BaseFunction):
             "input": {
                 "description": description,
                 "is_active": is_active,
+                "is_public": is_public,
                 "driver": driver,
                 "driver_opts": None if driver_opts is None else json.dumps(driver_opts),
                 "scheduler": scheduler,
                 "scheduler_opts": None if scheduler_opts is None else json.dumps(scheduler_opts),
                 "use_host_network": use_host_network,
+                "wsproxy_addr": wsproxy_addr,
+                "wsproxy_api_token": wsproxy_api_token,
             },
         }
         data = await api_session.get().Admin._query(query, variables)
@@ -208,15 +218,13 @@ class ScalingGroup(BaseFunction):
         """
         Deletes an existing scaling group.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($name: String!) {
                 delete_scaling_group(name: $name) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"name": name}
         data = await api_session.get().Admin._query(query, variables)
         return data["delete_scaling_group"]
@@ -230,16 +238,14 @@ class ScalingGroup(BaseFunction):
         :param scaling_group: The name of a scaling group.
         :param domain: The name of a domain.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($scaling_group: String!, $domain: String!) {
                 associate_scaling_group_with_domain(
                         scaling_group: $scaling_group, domain: $domain) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"scaling_group": scaling_group, "domain": domain}
         data = await api_session.get().Admin._query(query, variables)
         return data["associate_scaling_group_with_domain"]
@@ -253,16 +259,14 @@ class ScalingGroup(BaseFunction):
         :param scaling_group: The name of a scaling group.
         :param domain: The name of a domain.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($scaling_group: String!, $domain: String!) {
                 disassociate_scaling_group_with_domain(
                         scaling_group: $scaling_group, domain: $domain) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"scaling_group": scaling_group, "domain": domain}
         data = await api_session.get().Admin._query(query, variables)
         return data["disassociate_scaling_group_with_domain"]
@@ -275,15 +279,13 @@ class ScalingGroup(BaseFunction):
 
         :param domain: The name of a domain.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($domain: String!) {
                 disassociate_all_scaling_groups_with_domain(domain: $domain) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"domain": domain}
         data = await api_session.get().Admin._query(query, variables)
         return data["disassociate_all_scaling_groups_with_domain"]
@@ -297,16 +299,14 @@ class ScalingGroup(BaseFunction):
         :param scaling_group: The name of a scaling group.
         :param group_id: The ID of a group.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($scaling_group: String!, $user_group: UUID!) {
                 associate_scaling_group_with_user_group(
                         scaling_group: $scaling_group, user_group: $user_group) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"scaling_group": scaling_group, "user_group": group_id}
         data = await api_session.get().Admin._query(query, variables)
         return data["associate_scaling_group_with_user_group"]
@@ -320,16 +320,14 @@ class ScalingGroup(BaseFunction):
         :param scaling_group: The name of a scaling group.
         :param group_id: The ID of a group.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($scaling_group: String!, $user_group: String!) {
                 disassociate_scaling_group_with_user_group(
                         scaling_group: $scaling_group, user_group: $user_group) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"scaling_group": scaling_group, "user_group": group_id}
         data = await api_session.get().Admin._query(query, variables)
         return data["disassociate_scaling_group_with_user_group"]
@@ -342,15 +340,13 @@ class ScalingGroup(BaseFunction):
 
         :param group_id: The ID of a group.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($group_id: UUID!) {
                 disassociate_all_scaling_groups_with_group(user_group: $group_id) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"group_id": group_id}
         data = await api_session.get().Admin._query(query, variables)
         return data["disassociate_all_scaling_groups_with_group"]
