@@ -3,7 +3,7 @@ from __future__ import annotations
 import enum
 import textwrap
 import uuid
-from typing import Iterable, Sequence, Union
+from typing import Any, Iterable, Mapping, Sequence, Union
 
 from ai.backend.client.auth import AuthToken, AuthTokenTypes
 from ai.backend.client.output.fields import user_fields
@@ -26,11 +26,16 @@ _default_list_fields = (
     user_fields["role"],
     user_fields["username"],
     user_fields["email"],
+    user_fields["need_password_change"],
+    user_fields["status"],
+    user_fields["status_info"],
     user_fields["is_active"],
     user_fields["created_at"],
     user_fields["domain_name"],
     user_fields["groups"],
     user_fields["allowed_client_ip"],
+    user_fields["totp_activated"],
+    user_fields["sudo_session_enabled"],
 )
 
 _default_detail_fields = (
@@ -45,6 +50,8 @@ _default_detail_fields = (
     user_fields["role"],
     user_fields["groups"],
     user_fields["allowed_client_ip"],
+    user_fields["totp_activated"],
+    user_fields["sudo_session_enabled"],
 )
 
 
@@ -78,7 +85,12 @@ class User(BaseFunction):
     @api_function
     @classmethod
     async def authorize(
-        cls, username: str, password: str, *, token_type: AuthTokenTypes = AuthTokenTypes.KEYPAIR
+        cls,
+        username: str,
+        password: str,
+        *,
+        extra_args: Mapping[str, Any] = {},
+        token_type: AuthTokenTypes = AuthTokenTypes.KEYPAIR,
     ) -> AuthToken:
         """
         Authorize the given credentials and get the API authentication token.
@@ -89,14 +101,15 @@ class User(BaseFunction):
         of authentication methods.
         """
         rqst = Request("POST", "/auth/authorize")
-        rqst.set_json(
-            {
-                "type": token_type.value,
-                "domain": api_session.get().config.domain,
-                "username": username,
-                "password": password,
-            }
-        )
+        body = {
+            "type": token_type.value,
+            "domain": api_session.get().config.domain,
+            "username": username,
+            "password": password,
+        }
+        for k, v in extra_args.items():
+            body[k] = v
+        rqst.set_json(body)
         async with rqst.fetch() as resp:
             data = await resp.json()
             return AuthToken(
@@ -120,13 +133,11 @@ class User(BaseFunction):
         :param group: Fetch users in a specific group.
         :param fields: Additional per-user query fields to fetch.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             query($status: String, $group: UUID) {
                 users(status: $status, group_id: $group) {$fields}
             }
-        """
-        )
+        """)
         query = query.replace("$fields", " ".join(f.field_ref for f in fields))
         variables = {
             "status": status,
@@ -184,21 +195,17 @@ class User(BaseFunction):
         :param fields: Additional per-user query fields to fetch.
         """
         if email is None:
-            query = textwrap.dedent(
-                """\
+            query = textwrap.dedent("""\
                 query {
                     user {$fields}
                 }
-            """
-            )
+            """)
         else:
-            query = textwrap.dedent(
-                """\
+            query = textwrap.dedent("""\
                 query($email: String) {
                     user(email: $email) {$fields}
                 }
-            """
-            )
+            """)
         query = query.replace("$fields", " ".join(f.field_ref for f in fields))
         variables = {"email": email}
         data = await api_session.get().Admin._query(query, variables if email is not None else None)
@@ -219,21 +226,17 @@ class User(BaseFunction):
         :param fields: Additional per-user query fields to fetch.
         """
         if user_uuid is None:
-            query = textwrap.dedent(
-                """\
+            query = textwrap.dedent("""\
                 query {
                     user {$fields}
                 }
-            """
-            )
+            """)
         else:
-            query = textwrap.dedent(
-                """\
+            query = textwrap.dedent("""\
                 query($user_id: ID) {
                     user_from_uuid(user_id: $user_id) {$fields}
                 }
-            """
-            )
+            """)
         query = query.replace("$fields", " ".join(f.field_ref for f in fields))
         variables = {"user_id": str(user_uuid)}
         data = await api_session.get().Admin._query(
@@ -255,22 +258,22 @@ class User(BaseFunction):
         need_password_change: bool = False,
         description: str = "",
         allowed_client_ip: Iterable[str] = None,
+        totp_activated: bool = False,
         group_ids: Iterable[str] = None,
+        sudo_session_enabled: bool = False,
         fields: Iterable[FieldSpec | str] = None,
     ) -> dict:
         """
         Creates a new user with the given options.
         You need an admin privilege for this operation.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($email: String!, $input: UserInput!) {
                 create_user(email: $email, props: $input) {
                     ok msg user {$fields}
                 }
             }
-        """
-        )
+        """)
         default_fields = (
             user_fields["domain_name"],
             user_fields["email"],
@@ -290,8 +293,10 @@ class User(BaseFunction):
                 "need_password_change": need_password_change,
                 "description": description,
                 "domain_name": domain_name,
+                "totp_activated": totp_activated,
                 "group_ids": group_ids,
                 "allowed_client_ip": allowed_client_ip,
+                "sudo_session_enabled": sudo_session_enabled,
             },
         }
         data = await api_session.get().Admin._query(query, variables)
@@ -311,22 +316,22 @@ class User(BaseFunction):
         need_password_change: bool = None,
         description: str = None,
         allowed_client_ip: Iterable[str] = None,
+        totp_activated: bool = False,
         group_ids: Iterable[str] = None,
+        sudo_session_enabled: bool = False,
         fields: Iterable[FieldSpec | str] = None,
     ) -> dict:
         """
         Update existing user.
         You need an admin privilege for this operation.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($email: String!, $input: ModifyUserInput!) {
                 modify_user(email: $email, props: $input) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {
             "email": email,
             "input": {
@@ -339,7 +344,9 @@ class User(BaseFunction):
                 "need_password_change": need_password_change,
                 "description": description,
                 "allowed_client_ip": allowed_client_ip,
+                "totp_activated": totp_activated,
                 "group_ids": group_ids,
+                "sudo_session_enabled": sudo_session_enabled,
             },
         }
         data = await api_session.get().Admin._query(query, variables)
@@ -351,15 +358,13 @@ class User(BaseFunction):
         """
         Inactivates an existing user.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($email: String!) {
                 delete_user(email: $email) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {"email": email}
         data = await api_session.get().Admin._query(query, variables)
         return data["delete_user"]
@@ -374,15 +379,13 @@ class User(BaseFunction):
         Shared virtual folder's ownership will be transferred to the requested admin.
         To delete shared folders as well, set ``purge_shared_vfolders`` to ``True``.
         """
-        query = textwrap.dedent(
-            """\
+        query = textwrap.dedent("""\
             mutation($email: String!, $input: PurgeUserInput!) {
                 purge_user(email: $email, props: $input) {
                     ok msg
                 }
             }
-        """
-        )
+        """)
         variables = {
             "email": email,
             "input": {
