@@ -124,6 +124,87 @@ class VFolder(BaseFunction):
             page_size=page_size,
         )
 
+    @api_function
+    @classmethod
+    async def paginated_own_list(
+        cls,
+        *,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+        page_offset: int = 0,
+        page_size: int = 20,
+        filter: str = None,
+        order: str = None,
+    ) -> PaginatedResult[dict]:
+        """
+        Fetches the list of own vfolders.
+
+        :param fields: Additional per-vfolder query fields to fetch.
+        """
+        return await fetch_paginated_result(
+            "vfolder_own_list",
+            {
+                "filter": (filter, "String"),
+                "order": (order, "String"),
+            },
+            fields,
+            page_offset=page_offset,
+            page_size=page_size,
+        )
+
+    @api_function
+    @classmethod
+    async def paginated_invited_list(
+        cls,
+        *,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+        page_offset: int = 0,
+        page_size: int = 20,
+        filter: str = None,
+        order: str = None,
+    ) -> PaginatedResult[dict]:
+        """
+        Fetches the list of invited vfolders.
+
+        :param fields: Additional per-vfolder query fields to fetch.
+        """
+        return await fetch_paginated_result(
+            "vfolder_invited_list",
+            {
+                "filter": (filter, "String"),
+                "order": (order, "String"),
+            },
+            fields,
+            page_offset=page_offset,
+            page_size=page_size,
+        )
+
+    @api_function
+    @classmethod
+    async def paginated_project_list(
+        cls,
+        *,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+        page_offset: int = 0,
+        page_size: int = 20,
+        filter: str = None,
+        order: str = None,
+    ) -> PaginatedResult[dict]:
+        """
+        Fetches the list of invited vfolders.
+
+        :param fields: Additional per-vfolder query fields to fetch.
+        """
+        return await fetch_paginated_result(
+            "vfolder_project_list",
+            {
+                "filter": (filter, "String"),
+                "order": (order, "String"),
+            },
+            fields,
+            page_offset=page_offset,
+            page_size=page_size,
+        )
+
     @api_function
     @classmethod
     async def list_hosts(cls):
@@ -231,7 +312,11 @@ class VFolder(BaseFunction):
                                     ) as pbar:
                                         loop = current_loop()
                                         writer_fut = loop.run_in_executor(
-                                            None, self._write_file, file_path, file_mode, q.sync_q
+                                            None,
+                                            self._write_file,
+                                            file_path,
+                                            file_mode,
+                                            q.sync_q,
                                         )
                                         await asyncio.sleep(0)
                                         max_attempts = 10
@@ -239,7 +324,9 @@ class VFolder(BaseFunction):
                                             try:
                                                 async for attempt in AsyncRetrying(
                                                     wait=wait_exponential(
-                                                        multiplier=0.02, min=0.02, max=5.0
+                                                        multiplier=0.02,
+                                                        min=0.02,
+                                                        max=5.0,
                                                     ),
                                                     stop=stop_after_attempt(max_attempts),
                                                     retry=retry_if_exception_type(TryAgain),
@@ -278,6 +365,7 @@ class VFolder(BaseFunction):
         relative_paths: Sequence[Union[str, Path]],
         *,
         basedir: Union[str, Path] = None,
+        dst_dir: Union[str, Path] = None,
         chunk_size: int = DEFAULT_CHUNK_SIZE,
         show_progress: bool = False,
         address_map: Optional[Mapping[str, str]] = None,
@@ -306,31 +394,28 @@ class VFolder(BaseFunction):
                             "but no url matches with any of them.\n",
                         )
 
-                download_url = URL(overriden_url).with_query(
-                    {
-                        "token": download_info["token"],
-                    }
-                )
+                params = {"token": download_info["token"]}
+                if dst_dir is not None:
+                    params["dst_dir"] = dst_dir
+                download_url = URL(overriden_url).with_query(params)
             await self._download_file(
                 file_path, download_url, chunk_size, max_retries, show_progress
             )
 
-    @api_function
-    async def upload(
+    async def _upload_files(
         self,
-        files: Sequence[Union[str, Path]],
-        *,
+        file_paths: Sequence[Path],
         basedir: Union[str, Path] = None,
+        dst_dir: Union[str, Path] = None,
         chunk_size: int = DEFAULT_CHUNK_SIZE,
         address_map: Optional[Mapping[str, str]] = None,
-        show_progress: bool = False,
     ) -> None:
         base_path = Path.cwd() if basedir is None else Path(basedir).resolve()
-        if basedir:
-            files = [basedir / Path(file) for file in files]
-        else:
-            files = [Path(file).resolve() for file in files]
-        for file_path in files:
+        for file_path in file_paths:
+            if file_path.is_dir():
+                raise BackendClientError(
+                    f"Failed to upload {file_path}. Use recursive option to upload directories."
+                )
             file_size = Path(file_path).stat().st_size
             rqst = Request("POST", "/folders/{}/request-upload".format(self.name))
             rqst.set_json(
@@ -350,11 +435,10 @@ class VFolder(BaseFunction):
                             "Overriding storage proxy addresses are given, "
                             "but no url matches with any of them.\n",
                         )
-                upload_url = URL(overriden_url).with_query(
-                    {
-                        "token": upload_info["token"],
-                    }
-                )
+                params = {"token": upload_info["token"]}
+                if dst_dir is not None:
+                    params["dst_dir"] = dst_dir
+                upload_url = URL(overriden_url).with_query(params)
             tus_client = client.TusClient()
             if basedir:
                 input_file = open(base_path / file_path, "rb")
@@ -368,15 +452,59 @@ class VFolder(BaseFunction):
                 upload_checksum=False,
                 chunk_size=chunk_size,
             )
-            return await uploader.upload()
+            await uploader.upload()
+            input_file.close()
+
+    async def _upload_recursively(
+        self,
+        source: Sequence[Path],
+        basedir: Union[str, Path] = None,
+        dst_dir: Union[str, Path] = None,
+        chunk_size: int = DEFAULT_CHUNK_SIZE,
+        address_map: Optional[Mapping[str, str]] = None,
+    ) -> None:
+        dir_list: list[Path] = []
+        file_list: list[Path] = []
+        base_path = Path.cwd() if basedir is None else Path(basedir).resolve()
+        for path in source:
+            if path.is_file():
+                file_list.append(path)
+            else:
+                await self._mkdir(path.relative_to(base_path))
+                dir_list.append(path)
+        await self._upload_files(file_list, basedir, dst_dir, chunk_size, address_map)
+        for dir in dir_list:
+            await self._upload_recursively(
+                list(dir.glob("*")), basedir, dst_dir, chunk_size, address_map
+            )
 
     @api_function
-    async def mkdir(
+    async def upload(
+        self,
+        sources: Sequence[Union[str, Path]],
+        *,
+        basedir: Union[str, Path] = None,
+        recursive: bool = False,
+        dst_dir: Union[str, Path] = None,
+        chunk_size: int = DEFAULT_CHUNK_SIZE,
+        address_map: Optional[Mapping[str, str]] = None,
+        show_progress: bool = False,
+    ) -> None:
+        if basedir:
+            src_paths = [basedir / Path(src) for src in sources]
+        else:
+            src_paths = [Path(src).resolve() for src in sources]
+        if recursive:
+            await self._upload_recursively(src_paths, basedir, dst_dir, chunk_size, address_map)
+        else:
+            await self._upload_files(src_paths, basedir, dst_dir, chunk_size, address_map)
+
+    async def _mkdir(
         self,
         path: Union[str, Path],
         parents: Optional[bool] = False,
         exist_ok: Optional[bool] = False,
-    ):
+    ) -> str:
         rqst = Request("POST", "/folders/{}/mkdir".format(self.name))
         rqst.set_json(
             {
@@ -388,6 +516,15 @@ class VFolder(BaseFunction):
         async with rqst.fetch() as resp:
             return await resp.text()
 
+    @api_function
+    async def mkdir(
+        self,
+        path: Union[str, Path],
+        parents: Optional[bool] = False,
+        exist_ok: Optional[bool] = False,
+    ) -> str:
+        return await self._mkdir(path, parents, exist_ok)
+
     @api_function
     async def rename_file(self, target_path: str, new_name: str):
         rqst = Request("POST", "/folders/{}/rename-file".format(self.name))
@@ -619,3 +756,16 @@ class VFolder(BaseFunction):
         )
         async with rqst.fetch() as resp:
             return await resp.json()
+
+    @api_function
+    @classmethod
+    async def change_vfolder_ownership(cls, vfolder: str, user_email: str):
+        rqst = Request("POST", "/folders/_/change-ownership")
+        rqst.set_json(
+            {
+                "vfolder": vfolder,
+                "user_email": user_email,
+            }
+        )
+        async with rqst.fetch() as resp:
+            return await resp.json()
