@@ -36,11 +36,10 @@ from yarl import URL
 
 from .auth import generate_signature
 from .exceptions import BackendAPIError, BackendClientError
-from .session import AsyncSession, BaseSession
+from .session import AsyncSession, BaseSession, api_session
 from .session import Session as SyncSession
-from .session import api_session
 
-log = logging.getLogger(__name__)
+log = logging.getLogger(__spec__.name)  # type: ignore[name-defined]
 
 __all__ = [
     "Request",
@@ -144,7 +143,7 @@ class Request:
         :param BaseSession session: The session where this request is executed on.
 
         :param str path: The query path. When performing requests, the version number
-                         prefix will be automatically perpended if required.
+                         prefix will be automatically prepended if required.
 
         :param RequestContent content: The API query body which will be encoded as
                                        JSON.
@@ -418,7 +417,6 @@ class Request:
 
 
 class AsyncResponseMixin:
-
     _session: BaseSession
     _raw_response: aiohttp.ClientResponse
 
@@ -437,7 +435,6 @@ class AsyncResponseMixin:
 
 
 class SyncResponseMixin:
-
     _session: BaseSession
     _raw_response: aiohttp.ClientResponse
 
@@ -575,6 +572,7 @@ class FetchContextManager:
     async def __aenter__(self) -> Response:
         max_retries = len(self.session.config.endpoints)
         retry_count = 0
+        raw_resp: Optional[aiohttp.ClientResponse] = None
         while True:
             try:
                 retry_count += 1
@@ -598,8 +596,9 @@ class FetchContextManager:
                     self.session.config.rotate_endpoints()
                     continue
             except aiohttp.ClientResponseError as e:
-                msg = "API endpoint response error.\n" "\u279c {!r}".format(e)
-                await raw_resp.__aexit__(*sys.exc_info())
+                msg = "API endpoint response error.\n\u279c {!r}".format(e)
+                if raw_resp is not None:
+                    await raw_resp.__aexit__(*sys.exc_info())
                 raise BackendClientError(msg) from e
             finally:
                 self.session.config.load_balance_endpoints()
@@ -739,7 +738,7 @@ class WebSocketContextManager:
                     self.session.config.rotate_endpoints()
                     continue
             except aiohttp.ClientResponseError as e:
-                msg = "API endpoint response error.\n" "\u279c {!r}".format(e)
+                msg = "API endpoint response error.\n\u279c {!r}".format(e)
                 raise BackendClientError(msg) from e
             else:
                 break
@@ -767,7 +766,6 @@ class SSEMessage:
 
 
 class SSEResponse(BaseResponse):
-
     __slots__ = (
         "_auto_reconnect",
         "_retry",
@@ -850,7 +848,6 @@ class SSEResponse(BaseResponse):
 
 
 class SSEContextManager:
-
     __slots__ = (
         "session",
         "rqst_ctx_builder",
@@ -903,7 +900,7 @@ class SSEContextManager:
                     self.session.config.rotate_endpoints()
                     continue
             except aiohttp.ClientResponseError as e:
-                msg = "API endpoint response error.\n" "\u279c {!r}".format(e)
+                msg = "API endpoint response error.\n\u279c {!r}".format(e)
                 raise BackendClientError(msg) from e
             finally:
                 self.session.config.load_balance_endpoints()
